/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch/
 *
 */
 
 /**
 * @file bt_semaphore.h
 *
 * \author 14:50:09 Mathias Payer <payerm@student.ethz.ch>
 * 
 * \date 2004/05/07
 * 
 * @brief Implements mutex and bt_semaphores
 */

#ifndef _BT_BT_SEMAPHORE_H_
#define _BT_BT_SEMAPHORE_H_


/**
 * @struct bt_semaphore
 * @brief Represents a bt_semaphore, a number of ressources that
 * can be shared by many threads
*/
struct bt_semaphore {
	HANDLE queue;
	int counter;
};

struct _bt_semaphore_list {
    struct bt_semaphore * sema;
    struct _bt_semaphore_list * next;
};    

/*******************************************************************************
 * _bt_semaphore_init
 ******************************************************************************/
/**
 * Initializes a bt_semaphore
 * @param sema Pointer to the bt_semaphore
 * @param initial Initial value of the semaphore
 */
void _bt_semaphore_init(struct bt_semaphore *sema, int initial);


/*******************************************************************************
 * _bt_semaphore_trywait
 ******************************************************************************/
/**
 * If there are some ressources left, return one, if all ressources are taken, 
 * return an error.
 * @param sema Pointer to the bt_semaphore
 */
int _bt_semaphore_trywait(struct bt_semaphore *sema);


/*******************************************************************************
 * _bt_semaphore_wait
 ******************************************************************************/
/**
 * If there are some ressources left, return one, if all ressources are taken, 
 * wait until a ressource gets freed
 * @param sema Pointer to the bt_semaphore
 */
int _bt_semaphore_wait(struct bt_semaphore *sema);


/*******************************************************************************
 * _bt_semaphore_post
 ******************************************************************************/
/**
 * Increases the number of free ressources by 1
 * @param sema Pointer to the bt_semaphore
 */
int _bt_semaphore_post(struct bt_semaphore *sema);


/*******************************************************************************
 * _bt_semaphore_postval
 ******************************************************************************/
/**
 * Set the number of free ressources to val
 * @param sema Pointer to the bt_semaphore
 * @param val Number of free ressources
 */
int _bt_semaphore_postval(struct bt_semaphore *sema, int val);


/*******************************************************************************
 * _bt_semaphore_postinc
 ******************************************************************************/
/**
 * Increase the number of free ressources by val
 * @param sema Pointer to the bt_semaphore
 * @param val Number of additional free ressources
*/
int _bt_semaphore_postinc(struct bt_semaphore *sema, int val);


/*******************************************************************************
 * bt_semaphore_register_cmds
 ******************************************************************************/
/**
 * Prints out the current value of all bt semaphores on the terminal
*/
void bt_semaphore_register_cmds(void);

#endif
