/*
 * Copyright (C) 2005 Mario Strasser <mast@gmx.net>,
 *                    Swiss Federal Institute of Technology (ETH) Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: btn-timer.h,v 1.1 2005/08/15 10:01:45 mast2 Exp $
 */

/**
 * @file timer.h
 * @author Mario Strasser <mast@gmx.net>
 * @brief Implementation of an asynchronous (realtime) timer.
 */

#ifndef _TIMER_H_
#define _TIMER_H_

#include <sys/types.h>

/**
 * Starts the system timer 1 and registers a dedicated callback function.
 * The resolution of the timer is 10ms.
 * 
 * @param ms [in] timer interval in milliseconds
 * @param args [in] function to be called on each timer interval
 */
void btn_timer_start(u_long ms, void(*callback)(void *), void *arg);

/**
 * Stops the system timer 1.
 */
void btn_timer_stop(void);

/**
 * Indicates whether the timer is running or not.
 *
 * @return true (1) if the timer is running, false (0) otherwise.
 */
int btn_timer_is_running(void);

#endif /* _TIMER_H_ */
